/************************************************************************
 *
 *  Module:     NETUSBtest.c
 *  Long name:    
 *
 *  Runtime Env.: Linux
 *  Author(s):    gde (net gmbh)
 *  Company:      Net GmbH, Finning    http://www.net-gmbh.com
 *
 *  minimal ICube example
 *
 *
 ************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <string.h> 
#include <signal.h>
#include <stdlib.h>

#include "NETUSBCAM_API.h"
#include "ICubeDefines.h"
 
//---------------------------------------------------------------------------
// PROTOTYPES

void SignalHandler(int i);
void SaveRaw(unsigned char *buffer, unsigned int buffersize, const char* cName);

//---------------------------------------------------------------------------
// DEFINES

//#define  SAVE_RAW	// save a raw

//---------------------------------------------------------------------------
// GLOBALS
 
unsigned int nGoodCnt=0; 
unsigned int nBadCnt=0; 

//---------------------------------------------------------------------------
// Callback
//
int GetImage(void *buffer, unsigned int buffersize, void *context)
{
 
  if(buffersize==0){		// badframe arrived (this happens here, when (REG_CALLBACK_BR_FRAMES==1)
	nBadCnt++;
  }
  else				// good frame arrived
  { 
	nGoodCnt++; 
	#ifdef SAVE_RAW
	char buf[24];
	sprintf(buf,"Nr%d.raw",nGoodCnt);	
	SaveRaw((unsigned char*)buffer,buffersize,buf);
	#endif
  }
    
  printf("Got Image;  GoodFr: %d ,BadFr: %d , Size %d  \n",nGoodCnt,nBadCnt,buffersize);  
}
//---------------------------------------------------------------------------
//
int main()
{ 
	int result = 0;
	int CamIndex = 0; 	// (camera 0)
	
	printf("-----Hello ICube example----\n\n");

	result = NETUSBCAM_Init();		// look for ICubes
	if(result==0){
		printf("No device\n");
		return 0; }	

  	signal(SIGINT, SignalHandler);		// register signal handler for Ctrl+C

	result = NETUSBCAM_Open(CamIndex);	// open camera 
	if(result!=0){
		printf("Error: Open; Result = %d\n", result);
		return 0; }

	char cCamName[20];			// get camera model name
	result=NETUSBCAM_GetName(CamIndex,cCamName,20);
	if(result!=0){
		printf("Error: GetName; Result = %d\n", result);
		return 0; }
	printf("Model name: %s \n\n",cCamName);
	
	// set the camera clock lower, if a lot of bad frames arriving
	result = NETUSBCAM_SetCamParameter(CamIndex,REG_PLL,20);
	if(result!=0){
		printf("Error: REG_PLL; Result = %d\n", result);
		return 0; } 
		
	// if active, badframes are sent to the callback with buffersize = 0
	result = NETUSBCAM_SetCamParameter(CamIndex,REG_CALLBACK_BR_FRAMES,1);
	if(result!=0){
		printf("Error: REG_CALLBACK_BR_FRAMES; Result = %d\n", result);
		return 0; } 

	// set the callback to get the frame buffer
	result = NETUSBCAM_SetCallback(CamIndex,CALLBACK_RGB,&GetImage,NULL);
	if(result!=0){
		printf("Error: SetCallback; Result = %d\n", result);
		return 0; } 

	 // start streaming of camera
	result = NETUSBCAM_Start(CamIndex);
	if(result!=0){
		printf("Error: Start; Result = %d\n", result);
		return 0; }


	pause();		// wait for Ctrl+C
		 
	// stop streaming of camera
	result = NETUSBCAM_Stop(CamIndex);	
	if(result!=0){
		printf("Error: Stop; Result = %d\n", result);
		return 0; }
	
	// close camera
	result = NETUSBCAM_Close(CamIndex);	
	if(result!=0){
		printf("Error: Close; Result = %d\n", result);
		return 0; }

	return 0;
}
//---------------------------------------------------------------------------
// save data from callback
void SaveRaw(unsigned char *buffer, unsigned int buffersize, const char* cName)
{
   FILE *outfile = fopen( cName, "wb");
   if ( !outfile ){
		printf("Error fopen\n");
		return; }	
   fwrite (buffer,1,buffersize,outfile );
   fclose( outfile );	
}
//---------------------------------------------------------------------------
//
void SignalHandler(int i)
{
   signal(SIGINT, SignalHandler);
}

/***********************************EOF****************************************/

