/************************************************************************
 *
 *  Module:     NETUSBtest.c
 *  Long name:    
 *
 *  Runtime Env.: Linux
 *  Author(s):    gde (net gmbh)
 *  Company:      Net GmbH, Finning    http://www.net-gmbh.com
 *
 *  ROI SIMR TEST PROGRAM (Single Integration Multiple Roi)
 *
 *  for ICube models: 4133bu/cu
 *  for Api version 1.29 or above
 *
 ************************************************************************/

#include <unistd.h> 
#include <stdio.h>
#include <string.h>  
#include <signal.h>
#include <stdlib.h>

#include "NETUSBCAM_API.h"
#include "ICubeDefines.h"  
  
//---------------------------------------------------------------------------
// PROTOTYPES

void SignalHandler(int i);
void SetRoiID(int nRoiID);
void CheckResolutionDelta(int nRes);
void SaveRaw(unsigned char *buffer, unsigned int buffersize, const char* cName);

//---------------------------------------------------------------------------
// DEFINES

#define TRIGGERMODE 		// camera runs in trigger mode
#define TRIGGER_TIMEOUT 1000 

#define ROI_DELTA	4	// roi sizes must be diviable by ROI_DELTA

//#define SAVE_BMP		// save a bmp
//#define SAVE_RAW  		// save a raw

//---------------------------------------------------------------------------
// GLOBALS

int CamIndex = 0; 

bool bImageSwTrig=false;	// callback flag

unsigned int nGoodCnt=0; 
unsigned int nBadCnt=0; 

int nXResSIMR;
int nYResSIMR; 

//---------------------------------------------------------------------------
// Callback
//
int GetImage(void *buffer, unsigned int buffersize, void *context)
{
 
  if(buffersize==0){		// badframe arrived (this happens here, when (REG_CALLBACK_BR_FRAMES==1)
	nBadCnt++;
  }
  else				// good frame arrived
  { 
	nGoodCnt++;
	char buf[24];
	#ifdef SAVE_RAW
	sprintf(buf,"Nr%d_%dx%d.raw",nGoodCnt,nXResSIMR,nYResSIMR);	
	SaveRaw((unsigned char*)buffer,buffersize,buf);
	#endif
	#ifdef SAVE_BMP
	sprintf(buf,"Nr%d_%dx%d.bmp",nGoodCnt,nXResSIMR,nYResSIMR);	
	NETUSBCAM_SaveToFile(CamIndex,buf);
	#endif
  }
  
  bImageSwTrig = true;
  
  printf("Got SIMR;  GoodFr: %d ,BadFr: %d , Size %d  \n\n",nGoodCnt,nBadCnt,buffersize);  
}
//---------------------------------------------------------------------------
//
int main()
{
	int i = 0 , result = 0;
	char strBuf[256];

	// 2 rois can be defined, which will be sent in one image

	int nXPos[2], nYPos[2];		// starting points of rois
	int nXRes[2], nYRes[2]; 	// sizes of rois
	int nCnt=0;
	
	nXPos[0] = 0;	// roi1 definitions
	nYPos[0] = 0;
	nXRes[0] = 512;
	nYRes[0] = 200;

	nXPos[1] = 612;	// roi2 definitions
	nYPos[1] = 300;
	nXRes[1] = 512;
	nYRes[1] = 200; 

	nXResSIMR = nXRes[0]+nXRes[1];	// sizes of returned frame
	nYResSIMR = nYRes[0]+nYRes[1]; 
	
	printf("----Single Integration Multiple Roi----\n\n");

	CheckResolutionDelta(nXRes[0]);
	CheckResolutionDelta(nYRes[1]);
	CheckResolutionDelta(nXRes[0]);
	CheckResolutionDelta(nYRes[1]);

	result = NETUSBCAM_Init();
	if(result==0){
		printf("No device\n");
		return 0; }	

	signal(SIGINT, SignalHandler);		// register signal handler for Ctrl+C

	result = NETUSBCAM_Open(CamIndex);	// open one camera (camera 0)
	if(result!=0){
		printf("Error: Open; Result = %d\n", result);
		return 0; }
  
	memset(strBuf, 0, sizeof(strBuf));	// check camera model
	result=NETUSBCAM_GetName(CamIndex,strBuf,20);
	if(result!=0){
		printf("Error: GetName; Result = %d\n", result);
		return 0; }

	printf("Model name: %s \n\n",strBuf);
	if(!((strcmp(strBuf,"NET 4133 BU")==0) || (strcmp(strBuf,"NET 4133 CU")==0)||
		(strcmp(strBuf,"NET3 4133 BU")==0) || (strcmp(strBuf,"NET3 4133 CU")==0)||
		(strcmp(strBuf,"NET 4133 IR")==0) || (strcmp(strBuf,"NET3 4133 IR")==0)||
		(strcmp(strBuf,"NET 4203 BU")==0) || (strcmp(strBuf,"NET 4203 CU")==0)||
		(strcmp(strBuf,"NET3 4203 BU")==0) || (strcmp(strBuf,"NET3 4203 CU")==0)
		)){
		printf("Error: this example only works with NET4133bu/cu cameras\n");
		NETUSBCAM_Close(CamIndex);
		return 0; }
		
 
	result = NETUSBCAM_SetCamParameter(CamIndex,REG_CALLBACK_BR_FRAMES,1);
	if(result!=0){
		printf("Error: REG_CALLBACK_BR_FRAMES; Result = %d\n", result);
		return 0; } 

	result = NETUSBCAM_SetCallback(CamIndex,CALLBACK_RGB,&GetImage,NULL);
	if(result!=0){
		printf("Error: SetCallback; Result = %d\n", result);
		return 0; } 
		
 
	bImageSwTrig = false;   // callback flag

	// ROI_ID_1_1 must be written first
	printf("Resolution: ROI_%d  XRes = %d, YRes = %d\n",ROI_ID_1_1,nXRes[0],nYRes[0]);
	SetRoiID(ROI_ID_1_1);
	result = NETUSBCAM_SetResolution(CamIndex,nXRes[0],nYRes[0],nXPos[0],nYPos[0]);
	if(result!=0){
		printf("Error: SetResolution ROI_%d; Result = %d\n",ROI_ID_1_1,result);
		return 0; }
	
	printf("Resolution: ROI_%d  XRes = %d, YRes = %d\n\n",ROI_ID_1_2,nXRes[1],nYRes[1]);
	SetRoiID(ROI_ID_1_2);
	result = NETUSBCAM_SetResolution(CamIndex,nXRes[1],nYRes[1],nXPos[1],nYPos[1]);
	if(result!=0){
		printf("Error: SetResolution ROI_%d; Result = %d\n",ROI_ID_1_2,result);
		return 0; }
			

#ifdef TRIGGERMODE
	result = NETUSBCAM_SetTrigger(CamIndex, TRIG_SW_START);
	if(result!=0){
		printf("Error: SetTrigger; Result = %d\n", result);
		return 0; }
#endif

	result = NETUSBCAM_Start(CamIndex);	// start streaming of camera
	if(result!=0){
		printf("Error: Start; Result = %d\n", result);
		return 0; }
 
#ifdef TRIGGERMODE
	result = NETUSBCAM_SetTrigger(CamIndex, TRIG_SW_DO);
	if(result!=0){
		printf("Error: TRIG_SW_DO; Result = %d\n", result);
		return 0; }
	if (result == 0){
		int nCnt = 0;
		while(bImageSwTrig == false){
			usleep(10*1000);	// 10 msec
			nCnt++;
			// 100 msec timeout (should be bigger than exposure time)
			if(nCnt >TRIGGER_TIMEOUT){
				printf("Timeout waiting for callback flag!\n"); 
				break;
			}
		}
	} 
#else
	pause();	// wait for Ctrl+C
#endif
	result = NETUSBCAM_Stop(CamIndex);	// stop streaming of camera
	if(result!=0){
		printf("Error: Stop; Result = %d\n", result);
		return 0; }

	result = NETUSBCAM_Close(CamIndex);	// close camera
	if(result!=0){
		printf("Error: Close; Result = %d\n", result);
		return 0; }

	return 0;
}
//---------------------------------------------------------------------------
//
void CheckResolutionDelta(int nRes)
{
	int nTeiler = nRes % ROI_DELTA;		
	if(nTeiler!=0){		
		printf("Warning: resolution (%d) not dividable by ROI_DELTA \n",nRes);	
		printf("Warning: NETUSBCAM_SetResolution() will return error\n");	
	}
}
//---------------------------------------------------------------------------
//
void SignalHandler(int i)
{
   signal(SIGINT, SignalHandler);
}
//---------------------------------------------------------------------------
//
void SetRoiID(int nRoiID)
{
	int result = 0;	
	result = NETUSBCAM_SetCamParameter(CamIndex,REG_ROI_ID,nRoiID);
	if(result!=0)
		printf("Error: SetRoiID; Result = %d\n", result);
}
//---------------------------------------------------------------------------
//
void SaveRaw(unsigned char *buffer, unsigned int buffersize, const char* cName)
{
   FILE *outfile = fopen( cName, "wb");
   if ( !outfile ){
		printf("Error fopen\n");
		return; }	
   fwrite (buffer,1,buffersize,outfile );
   fclose( outfile );	
}

/***********************************EOF****************************************/

