#ifndef __ICUBEAPI_H__
#define __ICUBEAPI_H__

#include "ICubeDefines.h"


typedef int (*LPICubeInit)();
typedef LPICubeInit PICubeInit;

typedef int (*LPICubeGetApiVersion)(char* version, unsigned int length);
typedef LPICubeGetApiVersion PICubeGetApiVersion;

typedef int (*LPICubeOpen)(unsigned int deviceIndex);
typedef LPICubeOpen PICubeOpen;

typedef int (*LPICubeClose)(unsigned int deviceIndex);
typedef LPICubeClose PICubeClose;

typedef int (*LPICubeStart)(unsigned int deviceIndex);
typedef LPICubeStart PICubeStart;

typedef int (*LPICubeStop)(unsigned int deviceIndex);
typedef LPICubeStop PICubeStop;

typedef int (*LPICubeGetName)(unsigned int deviceIndex, char* name, unsigned int length);
typedef LPICubeGetName PICubeGetName;

typedef int (*LPICubeGetModeList)(unsigned int deviceIndex, unsigned int* length, unsigned int* modesList);
typedef LPICubeGetModeList PICubeGetModeList;

typedef int (*LPICubeSetMode)(unsigned int deviceIndex, unsigned int mode);
typedef LPICubeSetMode PICubeSetMode;

typedef int (*LPICubeGetMode)(unsigned int deviceIndex, unsigned int *mode);
typedef LPICubeGetMode PICubeGetMode;

typedef int (*LPICubeSetCallback)(unsigned int deviceIndex, int mode, int (*pCallbackFunc)(void* buffer, unsigned int bufferSize, void* context), void* context);
typedef LPICubeSetCallback PICubeSetCallback;

typedef int (*LPICubeGetBinSkipList)(unsigned int deviceIndex, unsigned int type, unsigned int* length, unsigned int* modesList);
typedef LPICubeGetBinSkipList PICubeGetBinSkipList;

typedef int (*LPICubeSetBinSkip)(unsigned int deviceIndex, int nFaktor, int nMode);
typedef LPICubeSetBinSkip PICubeSetBinSkip;

typedef int (*LPICubeGetResolution)(unsigned int deviceIndex, int* nXRes, int* nYRes, int* nXPos, int* nYPos);
typedef LPICubeGetResolution PICubeGetResolution;

typedef int (*LPICubeSetResolution)(unsigned int deviceIndex, int nXRes, int nYRes, int nXPos, int nYPos);
typedef LPICubeSetResolution PICubeSetResolution;

typedef int (*LPICubeGetSize)(unsigned int deviceIndex, int* nXRes, int* nYRes);
typedef LPICubeGetSize PICubeGetSize;

typedef int (*LPICubeGetBinSkip)(unsigned int deviceIndex,  int* nFaktor, int* nMode);
typedef LPICubeGetBinSkip PICubeGetBinSkip;

typedef int (*LPICubeGetResolutionRange)(unsigned int deviceIndex,ROI_RANGE_PROPERTY *property);
typedef LPICubeGetResolutionRange PICubeGetResolutionRange;

typedef int (*LPICubeSetCamParameter)(unsigned int deviceIndex,int Type,unsigned long Value);
typedef LPICubeSetCamParameter PICubeSetCamParameter;

typedef int (*LPICubeGetCamParameter)(unsigned int deviceIndex,int Type,unsigned long* Value);
typedef LPICubeGetCamParameter PICubeGetCamParameter;

typedef int (*LPICubeGetCamParameterRange)(unsigned int deviceIndex,int Type,PARAM_PROPERTY *property);
typedef LPICubeGetCamParameterRange PICubeGetCamParameterRange;

typedef int (*LPICubeGetParamAuto)(unsigned int deviceIndex,int Type,int* bAuto);
typedef LPICubeGetParamAuto PICubeGetParamAuto;

typedef int (*LPICubeSetParamAuto)(unsigned int deviceIndex,int Type,int bAuto);
typedef LPICubeSetParamAuto PICubeSetParamAuto;

typedef int (*LPICubeSetParamAutoDef)(unsigned int deviceIndex,int Type);
typedef LPICubeSetParamAutoDef PICubeSetParamAutoDef;

typedef int (*LPICubeSetParamOnePush)(unsigned int deviceIndex,int Type);
typedef LPICubeSetParamOnePush PICubeSetParamOnePush;

typedef int (*LPICubeGetFWVersion)(unsigned int deviceIndex,char* version, unsigned int length);
typedef LPICubeGetFWVersion PICubeGetFWVersion;

typedef int (*LPICubeGetTrigger)(unsigned int deviceIndex,int* nMode);
typedef LPICubeGetTrigger PICubeGetTrigger;

typedef int (*LPICubeSetTrigger)(unsigned int deviceIndex,int nMode);
typedef LPICubeSetTrigger PICubeSetTrigger;

typedef int (*LPICubeSaveToFile)(unsigned int deviceIndex,const char* name);
typedef LPICubeSaveToFile PICubeSaveToFile;

typedef int (*LPICubeGetBrokenFrames)(unsigned int deviceIndex,unsigned int* pnFrames);
typedef LPICubeGetBrokenFrames PICubeGetBrokenFrames;

typedef int (*LPICubeGetGoodFrames)(unsigned int deviceIndex,unsigned int* pnFrames);
typedef LPICubeGetGoodFrames PICubeGetGoodFrames;

//////////////////////////////////////////////////////////////////////////////'
 //
 //          Exposure functions 
 //          This functions can be used alternativly to parameter functions
 //          for manipulating exposure only. Parameter Value is in milli-seconds
 //          Be aware that every change of clock
 //          and width will lead to different exposure values
 //
 //////////////////////////////////////////////////////////////////////////////

typedef int (*LPICubeSetExposure)(unsigned int deviceIndex,float Value);
typedef LPICubeSetExposure PICubeSetExposure;

typedef int (*LPICubeGetExposure)(unsigned int deviceIndex,float* Value);
typedef LPICubeGetExposure PICubeGetExposure;

typedef int (*LPICubeGetExposureRange)(unsigned int deviceIndex,PARAM_PROPERTY_f *property);
typedef LPICubeGetExposureRange PICubeGetExposureRange;


extern bool LoadICubeAPI(void);
extern void UnloadICubeAPI(void);

extern PICubeInit 			ICubeInit;
extern PICubeGetApiVersion 	ICubeGetApiVersion; 
extern PICubeOpen 			ICubeOpen;
extern PICubeClose 			ICubeClose;
extern PICubeStart 			ICubeStart;
extern PICubeStop 			ICubeStop;
extern PICubeGetName 		ICubeGetName;
extern PICubeGetModeList 	ICubeGetModeList;
extern PICubeSetMode 		ICubeSetMode;
extern PICubeGetMode 		ICubeGetMode;
extern PICubeSetCallback 	ICubeSetCallback;
extern PICubeGetBinSkipList ICubeGetBinSkipList;
extern PICubeSetBinSkip 	ICubeSetBinSkip;
extern PICubeGetResolution 	ICubeGetResolution;
extern PICubeSetResolution 	ICubeSetResolution;
extern PICubeGetSize 		ICubeGetSize;
extern PICubeGetBinSkip 	ICubeGetBinSkip;
extern PICubeGetResolutionRange ICubeGetResolutionRange;
extern PICubeSetCamParameter ICubeSetCamParameter;
extern PICubeGetCamParameter ICubeGetCamParameter;
extern PICubeGetCamParameterRange ICubeGetCamParameterRange;
extern PICubeGetParamAuto 	ICubeGetParamAuto;
extern PICubeSetParamAuto 	ICubeSetParamAuto;
extern PICubeSetParamAutoDef 	ICubeSetParamAutoDef;
extern PICubeSetParamOnePush ICubeSetParamOnePush;
extern PICubeGetFWVersion 	ICubeGetFWVersion;
extern PICubeGetTrigger 	ICubeGetTrigger;
extern PICubeSetTrigger 	ICubeSetTrigger;
extern PICubeSaveToFile 	ICubeSaveToFile;
extern PICubeGetBrokenFrames ICubeGetBrokenFrames;
extern PICubeGetGoodFrames ICubeGetGoodFrames;
extern PICubeSetExposure 	ICubeSetExposure;
extern PICubeGetExposure 	ICubeGetExposure;
extern PICubeGetExposureRange ICubeGetExposureRange;

//
#endif // __ICUBEAPI_H__
