#include "VideoControlsDlg.h"
#include "ICubeAPI.h"
#include "ICubeDefines.h"
#include <QMessageBox>
#include "dialogimpl.h"

extern int nCurCam;


static struct symbolic_list mflist[] = {
	{ "30%" },
	{ "60%" },
	{ "100%" },
	{ NULL }
}; 

static struct symbolic_list shutterlist[] = {
	{ "Rolling(ERS)" },
	{ "Global reset" },
	{ "Global" },
	{ NULL }
}; 
//------------------------------------------------------------------------
//
#define INITCTRLMSG(a) if(stat!=0){	\
		QMessageBox::information(this,"Error: Init",a,QMessageBox::Ok);\
		return;\
	}

//------------------------------------------------------------------------
//
#define CTRLMSG(a) if(stat!=0){	\
		QMessageBox::information(this,"Error:Set",a,QMessageBox::Ok);\
		return;\
	}

//---------------------------------------------------------------------------
// Get and show exposure value is in milli-seconds
// Be aware, that every change of clock
// and width will lead to different exposure values.
// 
void VideoControlsDlg::ShowExposureMsec(void)
{
	if(checkBoxAutoExp->checkState())	// do not in auto exposure mode
		EditExposure->setText("auto");	
	else
	{
		float value;
		char buf[15];
	    int stat=ICubeGetExposure(nCurCam,&value);	// get msec value
	    CTRLMSG("Get Exposure"); 	
	    sprintf(buf,"%.1f",value);
	    EditExposure->setText(buf);					// show the value		
	}
}
//---------------------------------------------------------------------------
//
void VideoControlsDlg::ShowGammaDouble(unsigned int nGamma)
{
	char buf[15];
    double dGamma=(float)((float)nGamma/(float)DEFAULT_GAMMA);	
    sprintf(buf,"%.3f",dGamma);	
	EditGamma->setText(buf);	
}
//------------------------------------------------------------------------
//
int VideoControlsDlg::InitControl(int nType,QSlider *pSlider,QLineEdit *pEdit,QAbstractButton *pAuto,QAbstractButton *pOnePush)
{
unsigned long nVal;
int bAuto;
PARAM_PROPERTY param_prop;

    if(ICubeGetCamParameterRange(nCurCam,nType,&param_prop)!= 0 )
		return 1;

	if(param_prop.bEnabled)				// Control is supported
	{
		if(param_prop.bAuto)			// Control supports auto-mode
		{
			pAuto->setEnabled(true);						
			if(ICubeGetParamAuto(nCurCam,nType,&bAuto)!= 0 )
				return 1;			
			pAuto->setChecked(bAuto);
		}
		
		if(ICubeGetCamParameter(nCurCam,nType,&nVal)!= 0 )
			return 1;

		if(pSlider!=NULL)				// Control has Slider
		{
			pSlider->setEnabled(true);
			pSlider->setRange(param_prop.nMin,param_prop.nMax);
			pSlider->setValue(nVal);
			
			if(nType==REG_GAMMA)
				ShowGammaDouble(nVal);	// convert the value and show
			else if(nType==REG_EXPOSURE_TIME)
				ShowExposureMsec();		// get the value in msec. and show
			else
				pEdit->setText(tr("%1").arg(nVal));
		}

		if(param_prop.bOnePush)			// Control supports one push-mode
		{
			pOnePush->setEnabled(true);
		}
	}

return 0;
}
//---------------------------------------------------------------------
//
int VideoControlsDlg::SetControlToDefault(int nType,QSlider *pSlider,QLineEdit *pEdit,QAbstractButton *pAuto)
{
PARAM_PROPERTY param_prop;

    if (ICubeGetCamParameterRange(nCurCam,nType,&param_prop)!= 0 )
		return 1;
 
	if(param_prop.bEnabled)				// Control is supported
	{
		if(param_prop.bAuto)			// Control supports auto-mode
		{			
			// Set auto-parameter to default			
			if (ICubeSetParamAutoDef(nCurCam,nType)!= 0 )
				return 1;
			// Get the value for the checkbox	
			int nAuto;	
			if (ICubeGetParamAuto(nCurCam,nType,&nAuto)!= 0 )
				return 1;
	
			pAuto->setChecked(nAuto);			
		}

		if(pSlider!=NULL)				// Control has Slider
		{
			if (ICubeSetCamParameter(nCurCam,nType,param_prop.nDef)!= 0 )
				return 1;
			
			pSlider->setValue(param_prop.nDef);
			
			if(nType==REG_GAMMA)
				ShowGammaDouble(param_prop.nDef);
			else if(nType==REG_EXPOSURE_TIME)
				ShowExposureMsec();		// get the adjusted value in msec. and show			
			else			
				pEdit->setText(tr("%1").arg(param_prop.nDef));
		}
	}
return 0;
}
//---------------------------------------------------------------------
//
int VideoControlsDlg::InitShutter(void)
{
	unsigned long nVal;
    unsigned int i;
	PARAM_PROPERTY param_prop;
    
	if (ICubeGetCamParameterRange(nCurCam,REG_SHUTTER,&param_prop)!= 0 )
		return 1;

	if(param_prop.bEnabled)				// Control is supported
	{	
		comboBoxShutter->clear();
		for(i=0;i<3;i++){
			comboBoxShutter->addItem(shutterlist[i].name,0);
		}  	  
		if (ICubeGetCamParameter (nCurCam,REG_SHUTTER,&nVal)!= 0 )
			return 1;
	    comboBoxShutter->setCurrentIndex(nVal);
   }
  	
return 0;  	
}
//---------------------------------------------------------------------
//
int VideoControlsDlg::InitMField(void)
{
	unsigned long nVal;
    unsigned int i;
	PARAM_PROPERTY param_prop;
    
	if (ICubeGetCamParameterRange(nCurCam,REG_MEASURE_FIELD_AE,&param_prop)!= 0 )
		return 1;

	if(param_prop.bEnabled)				// Control is supported
	{	
		comboBoxMField->clear();
		for(i=0;i<3;i++){
			comboBoxMField->addItem(mflist[i].name,0);
		}  	  
		if (ICubeGetCamParameter (nCurCam,REG_MEASURE_FIELD_AE,&nVal)!= 0 )
			return 1;
	    comboBoxMField->setCurrentIndex(nVal);
   }
  	
return 0;  	
}
//---------------------------------------------------------------------
//
int VideoControlsDlg::InitInvertPixel(void)
{
	unsigned long nVal;
	PARAM_PROPERTY param_prop;
    
	if (ICubeGetCamParameterRange(nCurCam,REG_INVERT_PIXEL,&param_prop)!= 0 )
		return 1;

	if(param_prop.bEnabled)				// Control is supported
	{			 		
		if (ICubeGetCamParameter (nCurCam,REG_INVERT_PIXEL,&nVal)!= 0 )
			return 1;
		checkBoxInvertPixel->setEnabled(true);   
		checkBoxInvertPixel->setChecked(nVal);
   }
  	
return 0;  	
}
//---------------------------------------------------------------------
//
int VideoControlsDlg::InitDefectCor(void)
{
	unsigned long nVal;
	PARAM_PROPERTY param_prop;
    
	if (ICubeGetCamParameterRange(nCurCam,REG_DEFECT_COR,&param_prop)!= 0 )
		return 1;

	if(param_prop.bEnabled)				// Control is supported
	{			 		
		if (ICubeGetCamParameter (nCurCam,REG_DEFECT_COR,&nVal)!= 0 )
			return 1;
		checkBoxDefCor->setEnabled(true);   
		checkBoxDefCor->setChecked(nVal);
   }
  	
return 0;  	
}
//------------------------------------------------------------------------
// MField, Shutter
int VideoControlsDlg::SetComboControlsToDefault(int nType)
{
	PARAM_PROPERTY param_prop;
    
	if (ICubeGetCamParameterRange(nCurCam,nType,&param_prop)!= 0 )
		return 1;

	if(param_prop.bEnabled)				// Control is supported
	{
		if (ICubeSetCamParameter(nCurCam,nType,param_prop.nDef)!= 0 )
			return 1;
		bInit=false;	
		if(nType==REG_SHUTTER)
		{
			if (InitShutter()!= 0 )
				return 1;			
		}
		else
		{
			if (InitMField()!= 0 )
				return 1;			
		}
		bInit=true;	
	}
return 0;	
}
//------------------------------------------------------------------------
//
VideoControlsDlg::VideoControlsDlg( QWidget * parent, Qt::WFlags f) : QDialog(parent, f)
{
	setupUi(this);	
	connect(pushButtonDefault,SIGNAL(clicked()),SLOT(DefaultSlot()));
	connect(SliderBrightness,SIGNAL(valueChanged(int)),SLOT(sliderBrightnessSlot()) );
	connect(SliderContrast,SIGNAL(valueChanged(int)),SLOT(sliderConrastSlot()) );
	connect(SliderGamma,SIGNAL(valueChanged(int)),SLOT(sliderGammaSlot()) );
	connect(SliderExposure,SIGNAL(valueChanged(int)),SLOT(sliderExpSlot()) );
	connect(SliderExposureTarget,SIGNAL(valueChanged(int)),SLOT(sliderExpTargetSlot()) );
	connect(SliderPll,SIGNAL(valueChanged(int)),SLOT(sliderPllSlot()) );   
	connect(SliderGain,SIGNAL(valueChanged(int)),SLOT(sliderGainSlot()) );   
	connect(SliderBlackLevel,SIGNAL(valueChanged(int)),SLOT(sliderBlackLevelSlot()) );   
	connect(checkBoxAutoExp,SIGNAL(clicked()),SLOT(AutoExposureSlot()));
	connect(checkBoxBlacklevel,SIGNAL(clicked()),SLOT(AutoBlacklevelSlot()));
	connect(SliderRed,SIGNAL(valueChanged(int)),SLOT(sliderRedSlot()) );
	connect(SliderGreen,SIGNAL(valueChanged(int)),SLOT(sliderGreenSlot()) );
	connect(SliderBlue,SIGNAL(valueChanged(int)),SLOT(sliderBlueSlot()) );
	connect(pushButtonWB,SIGNAL(clicked()),SLOT(WbSlot()));
	connect(comboBoxMField,SIGNAL(currentIndexChanged (int)),SLOT(MFieldSlot(int)) );   	
	connect(comboBoxShutter,SIGNAL(currentIndexChanged (int)),SLOT(ShutterSlot(int)) ); 
	connect(checkBoxDefCor,SIGNAL(clicked()),SLOT(DefPixelCorSlot()));
	connect(checkBoxInvertPixel,SIGNAL(clicked()),SLOT(InvertPixelSlot()));
  		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::Activate(void)
{
	int stat;
	
	bInit=false;
		
	stat=InitControl(REG_BRIGHTNESS,SliderBrightness,EditBrightness);INITCTRLMSG("Brightness");  
	stat=InitControl(REG_CONTRAST,SliderContrast,EditContrast); INITCTRLMSG("Contrast"); 
	stat=InitControl(REG_GAMMA,SliderGamma,EditGamma); INITCTRLMSG("Gamma"); 
	stat=InitControl(REG_BLACKLEVEL,SliderBlackLevel,EditBlacklevel,checkBoxBlacklevel); INITCTRLMSG("BlckLev"); 
	stat=InitControl(REG_GAIN,SliderGain,EditGain); INITCTRLMSG("Gain"); 
	stat=InitControl(REG_EXPOSURE_TARGET,SliderExposureTarget,EditExposureTarget); INITCTRLMSG("ExpT"); 
	stat=InitControl(REG_EXPOSURE_TIME,SliderExposure,EditExposure,checkBoxAutoExp); INITCTRLMSG("Exp"); 
	stat=InitControl(REG_PLL,SliderPll,EditPll);INITCTRLMSG("Pll");
	stat=InitControl(REG_WHITE_BALANCE,NULL,NULL,NULL,pushButtonWB);INITCTRLMSG("WB");
	stat=InitControl(REG_RED,SliderRed,EditRed);INITCTRLMSG("Red");
	stat=InitControl(REG_GREEN,SliderGreen,EditGreen);INITCTRLMSG("Green");
	stat=InitControl(REG_BLUE,SliderBlue,EditBlue);INITCTRLMSG("Blue");
	stat=InitMField();INITCTRLMSG("InitMeasurementField");
	stat=InitShutter();INITCTRLMSG("InitShutter");
	stat=InitDefectCor();INITCTRLMSG("Defect pixel correction");
	stat=InitInvertPixel();INITCTRLMSG("InvertPixel");
		
	bInit=true;		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::AutoExposureSlot(void)
{
	unsigned int nAuto;
	if(checkBoxAutoExp->checkState())
	{
		nAuto=1;	
		EditExposure->setText("auto");	
		SliderExposure->setEnabled(false);
	}
	else
	{
		nAuto=0;
		SliderExposure->setEnabled(true);			
		ShowExposureMsec();
	}
	
	if(bInit==false)
		return;
			
	int stat=ICubeSetParamAuto(nCurCam,REG_EXPOSURE_TIME,nAuto);
	CTRLMSG("Set Auto Exposure"); 	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::AutoBlacklevelSlot(void)
{
	unsigned int nAuto;
	if(checkBoxBlacklevel->checkState())
		nAuto=1;	
	else
		nAuto=0;
	if(bInit==false)
		return;
	
	int stat=ICubeSetParamAuto(nCurCam,REG_BLACKLEVEL,nAuto);
	CTRLMSG("Set Auto Blacklevel"); 		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderExpSlot(void)
{	
	int nVal=SliderExposure->value();
		
	if(bInit==false)
		return;
		
	int stat=ICubeSetCamParameter(nCurCam,REG_EXPOSURE_TIME, nVal);
	CTRLMSG("Exposure"); 	
	ShowExposureMsec();			// get the adjusted value in msec. and show
			
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderExpTargetSlot(void)
{	
	int nVal=SliderExposureTarget->value();
	EditExposureTarget->setText(tr("%1").arg(nVal));
	
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_EXPOSURE_TARGET, nVal);
	CTRLMSG("Exposure Target"); 	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::DefaultSlot(void)
{
	int stat;

	stat=SetControlToDefault(REG_BRIGHTNESS,SliderBrightness,EditBrightness); CTRLMSG("Brightness"); 	
	stat=SetControlToDefault(REG_CONTRAST,SliderContrast,EditContrast);	CTRLMSG("Contrast"); 
	stat=SetControlToDefault(REG_GAMMA,SliderGamma,EditGamma); CTRLMSG("Gamma"); 
	stat=SetControlToDefault(REG_BLACKLEVEL,SliderBlackLevel,EditBlacklevel,checkBoxBlacklevel); CTRLMSG("BlackLevel"); 
	stat=SetControlToDefault(REG_EXPOSURE_TIME,SliderExposure,EditExposure,checkBoxAutoExp); CTRLMSG("Exposure"); 
	stat=SetControlToDefault(REG_EXPOSURE_TARGET,SliderExposureTarget,EditExposureTarget); CTRLMSG("Exposure Target"); 
	stat=SetControlToDefault(REG_GAIN,SliderGain,EditGain); CTRLMSG("Gain"); 
	stat=SetControlToDefault(REG_PLL,SliderPll,EditPll); CTRLMSG("Pll"); 
	stat=SetControlToDefault(REG_RED,SliderRed,EditRed); CTRLMSG("Red"); 
	stat=SetControlToDefault(REG_GREEN,SliderGreen,EditGreen); CTRLMSG("Green"); 
	stat=SetControlToDefault(REG_BLUE,SliderBlue,EditBlue); CTRLMSG("Blue"); 
	stat=SetComboControlsToDefault(REG_MEASURE_FIELD_AE);INITCTRLMSG("SetMeasurmentFieldToDefault");
	stat=SetComboControlsToDefault(REG_SHUTTER);INITCTRLMSG("SetShuttersToDefault");
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderPllSlot(void)
{	
	int nVal=SliderPll->value();
	EditPll->setText(tr("%1").arg(nVal));
	if(bInit==false)
		return;
				
	int stat=ICubeSetCamParameter(nCurCam,REG_PLL, nVal);
	if(stat==IC_WRONG_CLOCK_VAL)
	{
		SliderPll->setValue(NET_1044_BIN_MIN_CLOCK_VAL);
		EditPll->setText(tr("%1").arg(NET_1044_BIN_MIN_CLOCK_VAL));
		QMessageBox::information(this,"Error:","Pixelclock too low for this mode!",QMessageBox::Ok);
	}
	else
		CTRLMSG("Pll");	
	
	// pll value changes the exposure time, so get and show new value
	ShowExposureMsec();	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderBlackLevelSlot(void)
{	
	int nVal=SliderBlackLevel->value();
	EditBlacklevel->setText(tr("%1").arg(nVal));			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_BLACKLEVEL, nVal);
	CTRLMSG("Blacklevel");	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderGainSlot(void)
{	
	int nVal=SliderGain->value();
	EditGain->setText(tr("%1").arg(nVal));			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_GAIN , nVal);
	CTRLMSG("Gain");	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderBrightnessSlot(void)
{
	int nVal=SliderBrightness->value();
	EditBrightness->setText(tr("%1").arg(nVal));		
	if(bInit==false)
		return;
		
	int stat=ICubeSetCamParameter(nCurCam,REG_BRIGHTNESS,nVal);
	CTRLMSG("Brightness");		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderConrastSlot(void)
{
	int nVal=SliderContrast->value();
	EditContrast->setText(tr("%1").arg(nVal));			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_CONTRAST,nVal);
	CTRLMSG("Contrast");	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderGammaSlot(void)
{
	int nVal=SliderGamma->value();
	ShowGammaDouble(nVal);
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_GAMMA,nVal);
	CTRLMSG("Gamma");		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderRedSlot(void)
{
	int nVal=SliderRed->value();
	EditRed->setText(tr("%1").arg(nVal));			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_RED,nVal);
	CTRLMSG("Red");		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderGreenSlot(void)
{
	int nVal=SliderGreen->value();
	EditGreen->setText(tr("%1").arg(nVal));			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_GREEN,nVal);
	CTRLMSG("Green");		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::sliderBlueSlot(void)
{
	int nVal=SliderBlue->value();
	EditBlue->setText(tr("%1").arg(nVal));			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_BLUE,nVal);
	CTRLMSG("Blue");			
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::WbSlot(void)
{
	int stat=ICubeSetParamOnePush(nCurCam,REG_WHITE_BALANCE);
	CTRLMSG("White balance");		
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::MFieldSlot(int nMFieldIndex)
{
	if(bInit==false)
		return;
		
	int stat=ICubeSetCamParameter(nCurCam,REG_MEASURE_FIELD_AE,nMFieldIndex);
	CTRLMSG("Set MeasurementField");		
	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::ShutterSlot(int nShutterIndex)
{
	if(bInit==false)
		return;
		
	int stat=ICubeSetCamParameter(nCurCam,REG_SHUTTER,nShutterIndex);
	CTRLMSG("Set Shutter");		
	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::DefPixelCorSlot(void)
{
	unsigned int nVal;
	if(checkBoxDefCor->checkState())
		nVal=1;	
	else
		nVal=0;	
			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_DEFECT_COR,nVal);
	if(stat==IC_NOT_IF_STARTED)
	{
		InitDefectCor();	// restore check box value
		CTRLMSG("Set defect pixel correction only when video stopped!");
	}
	else			
		CTRLMSG("Set defect pixel correction");			
	
}
//------------------------------------------------------------------------
//
void VideoControlsDlg::InvertPixelSlot(void)
{
	unsigned int nVal;
	if(checkBoxInvertPixel->checkState())
		nVal=1;	
	else
		nVal=0;	
			
	if(bInit==false)
		return;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_INVERT_PIXEL,nVal);
	CTRLMSG("Set InvertPixel");	
	
}
//------------------------------------------------------------------------




