#include "ViewModeDlg.h"
#include "ICubeAPI.h"
#include "ICubeDefines.h"
#include <QMessageBox>
#include "dialogimpl.h"

extern int nCurCam;
extern int nBinMode;
extern int nSkipMode;
extern int nRoiMode;
extern int nXPos,nYPos,nXSize,nYSize;	


static struct symbolic_list binSkiplist[] = {
	{ "OFF" },
	{ "2x" },
	{ "4x" },
	{ NULL }
}; 

// Due to usb packet size, the Region of interest size has limitations.
// In this example, the horizontal and vertical Roi-size must be
// dividable by 4. For different Roi-sizes, see documentation.

#define ROI_RES_STEP_SIZE	4
#define ROI_POS_STEP_SIZE	2
//------------------------------------------------------------------------
//
ViewModeDlg::ViewModeDlg( QWidget * parent, Qt::WFlags f) : QDialog(parent, f)
{
	mParent=parent;
	
	setupUi(this);	
	connect(pushButtonROI,SIGNAL(clicked()),SLOT(SetROISlot()));
	connect(pushButtonBin,SIGNAL(clicked()),SLOT(SetBinSlot()));
	connect(pushButtonSkip,SIGNAL(clicked()),SLOT(SetSkipSlot()));
    connect(checkBoxRoi,SIGNAL(clicked()),SLOT(EnableROISlot()));
	
	connect(SliderXPos,SIGNAL(valueChanged(int)),SLOT(SliderXPosSlot()) );   
	connect(SliderYPos,SIGNAL(valueChanged(int)),SLOT(SliderYPosSlot()) );   
	connect(SliderXSize,SIGNAL(valueChanged(int)),SLOT(SliderXSizeSlot()) );   
	connect(SliderYSize,SIGNAL(valueChanged(int)),SLOT(SliderYSizeSlot()) );   	
	
	connect(this, SIGNAL(Restart()),mParent, SLOT(RestartSlot()));    				  
}
//------------------------------------------------------------------------
//
void ViewModeDlg::Activate(void)
{
	int stat=InitRoi();
	if(stat!=0)
		QMessageBox::information(this,"Error","InitRoi",QMessageBox::Ok);
		
    stat=InitBinSkip();
	if(stat!=0)
		QMessageBox::information(this,"Error","InitBinSkip",QMessageBox::Ok);	
}
//------------------------------------------------------------------------
//
int ViewModeDlg::InitBinSkip(void)
{
	int stat;
	unsigned int nList=MAX_MODE_COUNT;
    unsigned int i;
	
	// BIN
    stat=ICubeGetBinSkipList(nCurCam, MODE_BIN, &nList, nModBin);
    
	comboBoxBin->clear();
	comboBoxBin->addItem(binSkiplist[0].name,0);
	for(i=0;i<nList;i++){
		comboBoxBin->addItem(binSkiplist[nModBin[i]].name,nModBin[i]);
	}  
	 
	 comboBoxBin->setCurrentIndex(nBinMode);
	 
	// SKIP
	 stat=ICubeGetBinSkipList(nCurCam, MODE_SKIP, &nList, nModSkip);
    
	comboBoxSkip->clear();
	comboBoxSkip->addItem(binSkiplist[0].name,0);
	for(i=0;i<nList;i++){
		comboBoxSkip->addItem(binSkiplist[nModSkip[i]].name,nModSkip[i]);
	}  	  
	
	comboBoxSkip->setCurrentIndex(nSkipMode);
	
	return 0;
}
//------------------------------------------------------------------------
//
int ViewModeDlg::InitRoi(void)
{
	int stat;
	ROI_RANGE_PROPERTY	roi_range_prop;

	if(nRoiMode==1)
		checkBoxRoi->setChecked(true);		
	else
		checkBoxRoi->setChecked(false);
			
	stat=ICubeGetResolution(nCurCam,&nXSize,&nYSize,&nXPos,&nYPos);			
	stat=ICubeGetResolutionRange(nCurCam,&roi_range_prop);	

	EditXPos->setText(tr("%1").arg(nXPos));		
	EditYPos->setText(tr("%1").arg(nYPos));
	
	SliderXPos->setRange( roi_range_prop.nXMin,roi_range_prop.nXMax/ROI_POS_STEP_SIZE);
	SliderYPos->setRange( roi_range_prop.nYMin,roi_range_prop.nYMax/ROI_POS_STEP_SIZE );
	SliderXPos->setValue( nXPos/ROI_POS_STEP_SIZE );
	SliderYPos->setValue( nYPos/ROI_POS_STEP_SIZE );
	
	EditXSize->setText(tr("%1").arg(nXSize));		
	EditYSize->setText(tr("%1").arg(nYSize));
	
	SliderXSize->setRange( roi_range_prop.nXMin,roi_range_prop.nXMax/ROI_RES_STEP_SIZE );
	SliderYSize->setRange( roi_range_prop.nYMin,roi_range_prop.nYMax/ROI_RES_STEP_SIZE );
	SliderXSize->setValue( nXSize/ROI_RES_STEP_SIZE );
	SliderYSize->setValue( nYSize/ROI_RES_STEP_SIZE );	
				
	return 0;
}
//------------------------------------------------------------------------
//
void ViewModeDlg::SliderXPosSlot(void)
{
	int nNewVal=SliderXPos->value();
	nXPos=nNewVal*ROI_POS_STEP_SIZE;
	EditXPos->setText(tr("%1").arg(nXPos));
}
//------------------------------------------------------------------------
//
void ViewModeDlg::SliderYPosSlot(void)
{
	int nNewVal=SliderYPos->value();
	nYPos=nNewVal*ROI_POS_STEP_SIZE;
	EditYPos->setText(tr("%1").arg(nYPos));
}
//------------------------------------------------------------------------
//
void ViewModeDlg::SliderXSizeSlot(void)
{
	int nNewVal=SliderXSize->value();
	nXSize=nNewVal*ROI_RES_STEP_SIZE;
	EditXSize->setText(tr("%1").arg(nXSize));
}
//------------------------------------------------------------------------
//
void ViewModeDlg::SliderYSizeSlot(void)
{
	int nNewVal=SliderYSize->value();
	nYSize=nNewVal*ROI_RES_STEP_SIZE;
	EditYSize->setText(tr("%1").arg(nYSize));
}
//------------------------------------------------------------------------
//
void ViewModeDlg::EnableROISlot(void)
{
	if(checkBoxRoi->checkState())
		nRoiMode=1;	
	else
		nRoiMode=0;
}
//------------------------------------------------------------------------
//
void ViewModeDlg::SetROISlot(void)
{
	emit Restart();	
}
//------------------------------------------------------------------------
//
void ViewModeDlg::SetBinSlot(void)
{
	nBinMode=comboBoxBin->currentIndex();
	
	emit Restart();	
}
//------------------------------------------------------------------------
//
void ViewModeDlg::SetSkipSlot(void)
{
	nSkipMode=comboBoxSkip->currentIndex();
	
	emit Restart();	
}
//------------------------------------------------------------------------

