/***************************************************************************
 *   Copyright (C) 2013                                                    *
 *   NET GmbH                                                      		   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "dialogimpl.h"
#include "ViewModeDlg.h"

#include <QTimer>
#include <QMutex>
#include "ICubeAPI.h"
#include "ICubeDefines.h"


//#define DIRECT_PREVIEW	// enabled: Do preview direct from callback
							// disabled: Do preview from Timer (opt. Thread)
							
#define REFRESH_RATE	20	// in fps (only valid when DIRECT_PREVIEW is disabled)
							
//---------------------------------------------------------------------------
// GLOBALS

int nCurCam=0;

int nBinMode=0;
int nSkipMode=0;
int nRoiMode=0;
int nXPos=0,nYPos=0,nXSize=0,nYSize=0;	

   
QMutex mutex;

static struct symbolic_list modelist[] = {
	{ "320x240" },
	{ "640x480" },
	{ "752x480" },
	{ "800x600" },
	{ "1024x768" },
	{ "1280x1024"},
	{ "1600x1200"},
	{ "2048x1536"},
	{ "2592x1944",}, 
	{ "3840x2748",}, 	
	{ NULL }
}; 

//---------------------------------------------------------------------------
// Callback
 
int DialogImpl::ImageComplete(void* buffer, unsigned int bufferSize, void* context)
{
	DialogImpl* pDlg=(DialogImpl*)context;
	
	pDlg->cntFr++;			// FrameCounter
	pDlg->dFps=pDlg->FpsCounter->FpsCnt();		// fps Counter
	  		
	if(pDlg->bSaveRaw)		// Save the buffer to file
	{
		pDlg->bSaveRaw=false;	
		SaveRaw(buffer,bufferSize);
	}

	if(pDlg->bPreview)
	{
		QMutexLocker locker(&mutex);				
		if (pDlg->pBuffer != NULL)
		{						
			if (bufferSize <= pDlg->imgsize) 
				memcpy(pDlg->pBuffer,(unsigned char*)buffer,bufferSize);				
			#ifdef DIRECT_PREVIEW
				emit pDlg->PreviewFromCallback();
				pDlg->dFpsRefresh=pDlg->dFps;
			#endif					
		}					
	}

return 0;
}
//------------------------------------------------------------------------
//
void DialogImpl::PreviewSlot(void)
{
	
	if (pBuffer!=NULL) 
	{
		QMutexLocker locker(&mutex);		
		if(nGraphicMode==GRAPHIC_MODE_QT)
		{
			if(previewWindow->srcImage!=NULL)	 	
				previewWindow->previewLabel->setPixmap(QPixmap::fromImage(*previewWindow->srcImage));   
		}			
		else	
			GLpreviewWindow->updateWindow(pBuffer);

		#ifndef DIRECT_PREVIEW	
		dFpsRefresh=FpsRefreshCounter->FpsCnt();		// RefreshRate Counter
		#endif
	}	
}
//------------------------------------------------------------------------
//
int DialogImpl::initPreview(int nWidth,int nHeigth,int nCallbackformat)
{

	if(nGraphicMode==GRAPHIC_MODE_QT)
	{
		previewWindow->setGeometry(0,0,nWidth,nHeigth);
		previewWindow->previewLabel->setGeometry(0,0,nWidth,nHeigth);
		previewWindow->show();	
	}
	else
	{
		GLpreviewWindow =new glWindow();	
		GLpreviewWindow->setGeometry(0,0,nWidth,nHeigth);
		GLpreviewWindow->Activate(nWidth,nHeigth,nCallbackformat);
		GLpreviewWindow->show();
	}

	if(nCallbackformat==CALLBACK_RGB){
		imgsize = nHeigth*nWidth*3;
		if(pBuffer==NULL)	
			pBuffer = new unsigned char[imgsize];		
		
		if(nGraphicMode==GRAPHIC_MODE_QT){		
			if(previewWindow->srcImage==NULL)	
				previewWindow->srcImage = new QImage( pBuffer, nWidth, nHeigth, nWidth*3, QImage::Format_RGB888 );
			}
        }
	else if	((nCallbackformat==CALLBACK_RAW)||(nCallbackformat==CALLBACK_Y_8)){
		imgsize = nHeigth*nWidth;
		if(pBuffer==NULL)	
			pBuffer = new unsigned char[imgsize];	
				
		if(nGraphicMode==GRAPHIC_MODE_QT){		
			if(previewWindow->srcImage==NULL){			
				previewWindow->srcImage = new QImage( pBuffer, nWidth, nHeigth, nWidth, QImage::Format_Indexed8 ); 
				QVector<QRgb> colorTable;
	    		for (int i = 0; i < 256; i++)
	        		colorTable.push_back(QColor(i, i, i).rgb());
			    previewWindow->srcImage->setColorTable(colorTable);
	  		 }
 		 }
		}
	else 
		return 1;
		   
return 0;         
}
//------------------------------------------------------------------------
//
void DialogImpl::releasePreview(void)
{
	
	QMutexLocker locker(&mutex);
		 		
	if(pBuffer){
		delete [] pBuffer;
		pBuffer=NULL;
	}
	
	if(nGraphicMode==GRAPHIC_MODE_QT)
	{		
		if(previewWindow->srcImage){
				delete previewWindow->srcImage;
				previewWindow->srcImage=NULL;
			}		
		previewWindow->hide();
	}
	else
	{
		GLpreviewWindow->hide();
		GLpreviewWindow->close();
	}

}
//------------------------------------------------------------------------
//
DialogImpl::DialogImpl( QWidget * parent, Qt::WFlags f) : QDialog(parent, f)
{	
char cVersion[6];
char cName[30];

	setupUi(this);
	connect(pushButtonStart,SIGNAL(clicked()),SLOT(StartSlot()));
	connect(pushButtonStop,SIGNAL(clicked()),SLOT(StopSlot()));
	connect(pushButtonSaveRaw,SIGNAL(clicked()),SLOT(SaveRawSlot()));
	connect(pushButtonOpen,SIGNAL(clicked()),SLOT(OpenSlot()));
	connect(pushButtonClose,SIGNAL(clicked()),SLOT(CloseSlot()));
	connect(checkBoxPreview,SIGNAL(clicked()),SLOT(bPreviewSlot()));
	connect(comboBoxMode,SIGNAL(currentIndexChanged (int)),SLOT(ModeSlot(int)) );   
	connect(this, SIGNAL(PreviewFromCallback()),this, SLOT(PreviewSlot()));    				  
	connect(checkBoxFlipH,SIGNAL(clicked()),SLOT(FlipHSlot()));
	connect(checkBoxFlipV,SIGNAL(clicked()),SLOT(FlipVSlot()));
	connect(pushButtonVideoControls,SIGNAL(clicked()),SLOT(VideoControlSlot()));	
	connect(pushButtonViewMode,SIGNAL(clicked()),SLOT(ViewModeSlot()));         	    
    connect(radioButton8bit,SIGNAL(clicked()),SLOT(r8bitSlot()));	   
    connect(radioButton8bitY,SIGNAL(clicked()),SLOT(r8bitYSlot()));	      	           	    
    connect(radioButton24bit,SIGNAL(clicked()),SLOT(r24bitSlot()));	      	           	     	
    connect(radioButtonSWTrigger,SIGNAL(clicked()),SLOT(TrigSWSlot()));	      	    
    connect(radioButtonHWTrigger,SIGNAL(clicked()),SLOT(TrigHWSlot()));	      	    
	connect(checkBoxTrigger,SIGNAL(clicked()),SLOT(TrigEnSlot()));
	connect(pushButtonSWTrigger,SIGNAL(clicked()),SLOT(TrigDoSWSlot()));         	    
    connect(radioButtonGraphicQT,SIGNAL(clicked()),SLOT(GraphicQTSlot()));	      	    
    connect(radioButtonGraphicGL,SIGNAL(clicked()),SLOT(GraphicGLSlot()));	
    connect(pushButtonSaveRGB,SIGNAL(clicked()),SLOT(SaveRGBSlot()));	
	connect(comboBoxCam,SIGNAL(currentIndexChanged (int)),SLOT(CamSlot(int)) );   
	connect(SliderTriggerDelay,SIGNAL(valueChanged(int)),SLOT(sliderTriggerDelaySlot()) );
	connect(SliderStrobeDelay,SIGNAL(valueChanged(int)),SLOT(sliderStrobeDelaySlot()) );
	connect(SliderStrobeLength,SIGNAL(valueChanged(int)),SLOT(sliderStrobeLengthSlot()) );
	connect(checkBoxInvertHWTrigger,SIGNAL(clicked()),SLOT(TrigInvertHWSlot()));      	                 
	      	                  	                  	            	     	
	QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(updateCaption()));	// show fps,... in timer    
    timer->start(200);
    FpsCounter= new CFpsCounter();
    dFps=0;
#ifndef DIRECT_PREVIEW	
	QTimer *timerRefresh = new QTimer(this);
    connect(timerRefresh, SIGNAL(timeout()), this, SLOT(PreviewSlot()));// Timer to refresh preview	 						
	timerRefresh->start(1000/REFRESH_RATE);	
	FpsRefreshCounter= new CFpsCounter();
	dFpsRefresh=0;
#endif
    cntFr=0;
	g_iCurrentMode=0;

	previewWindow = new PreviewWindow(this);	// init external preview window (non gl)	
	viewModeDlg = new ViewModeDlg(this);
	videoControlsDlg = new VideoControlsDlg;

	previewWindow->srcImage=NULL;
	pBuffer=NULL;
	imgsize = 0;
          
	bSaveRaw=false;
	bPreview=true;
	bIsOpen=false;
	bStarted=false;
    nCallbackType=CALLBACK_RGB;
    nTriggerMode=TRIG_SW_START;
    nGraphicMode=GRAPHIC_MODE_QT;
     
	if(!LoadICubeAPI()){
		QMessageBox::information( 0,"Error"," lib not loaded ",0);	
		return;
	}
	
	int stat=ICubeGetApiVersion(cVersion, 6);
	if(stat!=0){
		QMessageBox::information( 0,"Error"," GetApiVersion failed",0);		
		return;
	}
	else
	{
		labelVers->setText(cVersion);
		int nVersLow=atoi(&cVersion[2]);
		if(nVersLow<17)	
			QMessageBox::information( 0,"Warning","This sample works only with API Version X.17 and above correctly",0);		
	}
		
			
	int camCount = ICubeInit();
	if (camCount<=0) {					// failed
		QMessageBox::information( 0,"Info"," No camera detected ",0);	
		return ;
		}	
		
	comboBoxCam->clear();		
	for(int i=0;i<camCount;i++)
	{
	stat=ICubeGetName(i,cName,20);
	if(stat!=0){
		QMessageBox::information( 0,"Error"," GetName failed",0);		
		return;
		}			
	else
		comboBoxCam->addItem(cName,i);	
	}	
}
//------------------------------------------------------------------------
//
DialogImpl::~DialogImpl()
{ }
//------------------------------------------------------------------------
// 
void DialogImpl::closeEvent(QCloseEvent *event)
 {
 	videoControlsDlg->hide();	// Close sub windows
	viewModeDlg->hide();		// Close sub windows
	CloseCam();
	UnloadICubeAPI();
 	event->accept();
 } 
//------------------------------------------------------------------------
//
void DialogImpl::OpenSlot()
{	
	char cName[20];
	unsigned int nList=MAX_MODE_COUNT,i;
	    
	g_iCurrentMode=0;
	
	int stat=ICubeOpen(nCurCam);
	if(stat!=0){
		QMessageBox::information( 0,"Error"," Open failed",0);	
		return;		
		}
		
	bIsOpen=true;
	
	stat=ICubeGetModeList(nCurCam,&nList, g_ModIndexCam);	// get the possible modes
	if(stat!=0){
		QMessageBox::information( 0,"Error"," GetModeList failed",0);		
		return;
		}	
		
	comboBoxMode->clear();
	for(i=0;i<nList;i++){
		comboBoxMode->addItem(modelist[g_ModIndexCam[i]].name,g_ModIndexCam[i]);
	}
	
	stat=ICubeGetMode(nCurCam,&g_iCurrentMode);		// get the curent mode
	if(stat!=0){
		QMessageBox::information( 0,"Error"," GetMode failed",0);		
		return;
		}		
		
	g_iCurrentMode=g_ModIndexCam[comboBoxMode->currentIndex()];
	
	stat=ICubeGetFWVersion(nCurCam,cName,8);
	if(stat!=0){
		QMessageBox::information( 0,"Error"," GetFWVersion failed",0);	
		return;
		}	
	labelFWVersion->setText(cName);
	
	stat=initTriggerControls(); // Trigger Delay,...
	if(stat!=0)
		QMessageBox::information( 0,"Error"," initTriggerControls failed",0);	
}
//------------------------------------------------------------------------
//
void DialogImpl::StartSlot()
{
	StartCam();
}
//------------------------------------------------------------------------
//
void DialogImpl::RestartSlot(void)
{
	if(!bStarted)
		return;		// if not started -> do nothing
		
	int stat;
	stat=StopCam();
	if(stat!=0){
		QMessageBox::information( 0,"Error"," Stop failed",0);
		return;	
	}
			
	StartCam();
}
//------------------------------------------------------------------------
//
void DialogImpl::StartCam(void)
{	
	int stat;
	int n_Width,n_Heigth;

	cntFr=0;					// Init Frame Counter
	FpsCounter->ResetFpsCnt();	// Init fps Counter
#ifndef DIRECT_PREVIEW		
	FpsRefreshCounter->ResetFpsCnt();	// Init RefreshRate Counter
#endif	

	if(bIsOpen)
	{		
		stat=ICubeSetMode(nCurCam,g_iCurrentMode);
		if ( stat != 0 ){
				QMessageBox::information( 0, "Error:", " SetMode failed", 0 );
				return;
		}
		
		if(nBinMode!=0)		// if binning is enabled
		{
			stat=ICubeSetBinSkip(nCurCam,nBinMode,MODE_BIN);
			if ( stat != 0 )
			{
				if(stat==IC_WRONG_CLOCK_VAL)			
					QMessageBox::information(this,"Error:","Pixelclock too low for this mode!",QMessageBox::Ok);			
				else
					QMessageBox::information( 0,"Error"," SetBin failed",0);					
				return;
			}		
		}

		if(nSkipMode!=0)		// if skipping is enabled
		{
			stat=ICubeSetBinSkip(nCurCam,nSkipMode,MODE_SKIP);
			if ( stat != 0 ){
					QMessageBox::information( 0,"Error"," SetSkip failed",0);
					return;
			}		
		}
		if(nRoiMode!=0)		// if roi is enabled
		{
			stat=ICubeSetResolution(nCurCam,nXSize,nYSize,nXPos,nYPos);
			if ( stat != 0 ){
					QMessageBox::information( 0,"Error"," SetResolution failed",0);
					return;
			}		
		}			
			
		stat=ICubeGetSize(nCurCam,&n_Width,&n_Heigth);
		if ( stat != 0 ){
			QMessageBox::information( 0,"Error"," GetSize failed",0);
			return;
		}
		char buf[10];
		sprintf(buf,"%dx%d",n_Width,n_Heigth);
		labelRes->setText(tr(buf));					// Show resolution

		if(bPreview){						
			stat=initPreview(n_Width,n_Heigth,nCallbackType);
			if(stat!=0){
				QMessageBox::information( 0,"Error"," initPreview failed",0);		
				return;
			}
		}
			
		stat = ICubeSetCallback(nCurCam, nCallbackType, &ImageComplete,this);
		if(stat!=0){
			QMessageBox::information( 0,"Error"," SetCallback failed",0);		
			return;
			}
			
		stat=ICubeStart(nCurCam);
		if(stat!=0)
			QMessageBox::information( 0,"Error"," Start failed",0);	
 
	bStarted=true;			
	}
	else
		QMessageBox::information( 0,"Error"," Interface not open",0);	
		
}
//------------------------------------------------------------------------
//
int DialogImpl::StopCam(void)
{
	int stat=ICubeStop(nCurCam);
		
	ICubeSetCallback(nCurCam, CALLBACK_RGB, NULL,NULL);
	
	if(bPreview)	
		releasePreview();
	
	bStarted=false;	
	
return stat;			
}
//------------------------------------------------------------------------
//
void DialogImpl::CloseCam(void)
{
int stat;

	if(bIsOpen){
		if(bStarted==true){
			stat=StopCam();
			if(stat!=0)
				QMessageBox::information( 0,"Error"," StopCam",0);
		}
		stat=ICubeClose(nCurCam);
		if(stat!=0)
			QMessageBox::information( 0,"Error"," Close",0);
		
		bIsOpen=false;

		comboBoxMode->clear();		
	}	
}
//------------------------------------------------------------------------
//
void DialogImpl::StopSlot(void)
{	
	int stat=StopCam();
	if(stat!=0)
		QMessageBox::information( 0,"Error"," StopCam",0);		
}
//------------------------------------------------------------------------
//
void DialogImpl::CloseSlot(void)
{	
	CloseCam();
}
//------------------------------------------------------------------------
//
void DialogImpl::bPreviewSlot(void)
{
	bPreview=checkBoxPreview->checkState();
}
//------------------------------------------------------------------------
//
void DialogImpl::SaveRawSlot(void)
{
	bSaveRaw=true;	// set flag for save in callback
}
//------------------------------------------------------------------------
//
void DialogImpl::updateCaption(void)
{
	labelFrames->setText(tr("%1").arg(cntFr));
	label_fps->setText(tr("%1").arg(dFps));
	label_fpsRefresh->setText(tr("%1").arg(dFpsRefresh));
	
	if(bStarted==true)
    {
        unsigned int brokenframes;
        ICubeGetBrokenFrames(nCurCam,&brokenframes);
		label_BadFrames->setText(tr("%1").arg(brokenframes));

        unsigned int goodframes;
        ICubeGetGoodFrames(nCurCam,&goodframes);
		label_GoodFrames->setText(tr("%1").arg(goodframes));
    }	
}
//------------------------------------------------------------------------
//
void DialogImpl::CamSlot(int nCamIndex)
{
	nCurCam=nCamIndex;
}
//------------------------------------------------------------------------
//
void DialogImpl::ModeSlot(int nModeIndex)
{
	g_iCurrentMode=g_ModIndexCam[nModeIndex];
}
//------------------------------------------------------------------------
//
void DialogImpl::VideoControlSlot(void)
{
	if(!bIsOpen){
		QMessageBox::information( 0,"Error"," Interface not open",0);
		return;
	}
	videoControlsDlg->show();
	videoControlsDlg->Activate();
}
//------------------------------------------------------------------------
//
void DialogImpl::ViewModeSlot(void)
{
	if(!bIsOpen){
		QMessageBox::information( 0,"Error"," Interface not open",0);
		return;
	}		
	viewModeDlg->show();
	viewModeDlg->Activate();
}
//------------------------------------------------------------------------
// FLip related
//------------------------------------------------------------------------
//
void DialogImpl::FlipHSlot(void)
{
	unsigned int nFlipState;
	
	if(checkBoxFlipH->checkState())
		nFlipState=1;	
	else
		nFlipState=0;
		
	int stat=ICubeSetCamParameter(nCurCam,REG_FLIPPED_H,nFlipState);
	if(stat!=0)
		QMessageBox::information( 0,"Error"," Set flip hor. failed",0);
}
//------------------------------------------------------------------------
//
void DialogImpl::FlipVSlot(void)
{
	unsigned int nFlipState;
	
	if(checkBoxFlipV->checkState())
		nFlipState=1;	
	else
		nFlipState=0;
	
	int stat=ICubeSetCamParameter(nCurCam,REG_FLIPPED_V,nFlipState);
	if(stat!=0)
		QMessageBox::information( 0,"Error"," Set flip ver. failed",0);		
}
//------------------------------------------------------------------------
// Bit Depth related
//------------------------------------------------------------------------
//
void DialogImpl::r8bitSlot(void)
{
	nCallbackType=CALLBACK_RAW;
}
//------------------------------------------------------------------------
//
void DialogImpl::r24bitSlot(void)
{
	nCallbackType=CALLBACK_RGB;
}
//------------------------------------------------------------------------
//
void DialogImpl::r8bitYSlot(void)
{
	nCallbackType=CALLBACK_Y_8;
}
//------------------------------------------------------------------------
// graphic related
//------------------------------------------------------------------------
//
void DialogImpl::GraphicQTSlot(void)
{
	nGraphicMode=GRAPHIC_MODE_QT;	// Non OpenGl Preview (preview in Qlabel)
}
//------------------------------------------------------------------------
//
void DialogImpl::GraphicGLSlot(void)
{
	nGraphicMode=GRAPHIC_MODE_GL;	// OpenGl Preview 
}
//------------------------------------------------------------------------
//
void DialogImpl::SaveRGBSlot(void)
{
	int stat=ICubeSaveToFile(nCurCam,"icube.bmp");	
	if(stat!=0)
		QMessageBox::information( 0,"Error"," SaveToFile failed",0);	
}
//------------------------------------------------------------------------
// Trigger related
//------------------------------------------------------------------------
// 
void DialogImpl::TrigEnSlot(void)
{
int stat;

	if(checkBoxTrigger->checkState())
	{
		stat=ICubeSetTrigger(nCurCam,nTriggerMode);		// Start selected trigger mode
		if(stat!=0){
			QMessageBox::information( 0,"Error"," Start Trigger failed",0);	
			return;
		}	
	}
	else
	{
		stat=ICubeSetTrigger(nCurCam,TRIG_STOP);		// Stop trigger
		if(stat!=0)
			QMessageBox::information( 0,"Error"," Stop Trigger failed",0);	
	}	
}
//------------------------------------------------------------------------
// 
void DialogImpl::TrigSWSlot(void)
{
	nTriggerMode=TRIG_SW_START;
}
//------------------------------------------------------------------------
// 
void DialogImpl::TrigHWSlot(void)
{
	nTriggerMode=TRIG_HW_START;
}
//------------------------------------------------------------------------
// 
void DialogImpl::TrigDoSWSlot(void)
{
	if(nTriggerMode==TRIG_HW_START)
	{
		QMessageBox::information( 0,"Error"," Wrong trigger mode",0);
		return;
	}
	int stat=ICubeSetTrigger(nCurCam,TRIG_SW_DO);	// new software trigger command
	if(stat!=0)
		QMessageBox::information( 0,"Error"," SetTrigger failed",0);				

}
//------------------------------------------------------------------------
//
void DialogImpl::TrigInvertHWSlot(void)
{
int stat;
	
	if(checkBoxInvertHWTrigger->checkState())
	{
		stat=ICubeSetCamParameter(nCurCam,REG_TRIGGER_INVERT, 1);		
		if(stat!=0){
			QMessageBox::information( 0,"Error"," Invert Trigger failed",0);	
			return;
		}	
	}
	else
	{
		stat=ICubeSetCamParameter(nCurCam,REG_TRIGGER_INVERT, 0);		
		if(stat!=0)
			QMessageBox::information( 0,"Error"," Invert Trigger failed",0);	
	}	
}
//------------------------------------------------------------------------
//
int DialogImpl::initTriggerControls(void)
{
	PARAM_PROPERTY param_prop;
	unsigned long nVal;
	
 	if(ICubeGetCamParameterRange(nCurCam,REG_TRIGGER_DELAY,&param_prop)!= 0 )
		return 1;
	if(ICubeGetCamParameter(nCurCam,REG_TRIGGER_DELAY,&nVal)!= 0 )
		return 1;

	SliderTriggerDelay->setRange(param_prop.nMin,param_prop.nMax);
	SliderTriggerDelay->setValue(nVal);
	EditTriggerDelay->setText(tr("%1").arg(nVal));	

	if(ICubeGetCamParameterRange(nCurCam,REG_STROBE_DELAY,&param_prop)!= 0 )
		return 1;
	if(ICubeGetCamParameter(nCurCam,REG_STROBE_DELAY,&nVal)!= 0 )
		return 1;

	SliderStrobeDelay->setRange(param_prop.nMin,param_prop.nMax);
	SliderStrobeDelay->setValue(nVal);
	EditStrobeDelay->setText(tr("%1").arg(nVal));	

	if(ICubeGetCamParameterRange(nCurCam,REG_STROBE_LENGHT,&param_prop)!= 0 )
		return 1;
	if(ICubeGetCamParameter(nCurCam,REG_STROBE_LENGHT,&nVal)!= 0 )
		return 1;

	SliderStrobeLength->setRange(param_prop.nMin,param_prop.nMax);
	SliderStrobeLength->setValue(nVal);
	EditStrobeLength->setText(tr("%1").arg(nVal));	

return 0;	
}
//------------------------------------------------------------------------
//
void DialogImpl::sliderTriggerDelaySlot(void)
{
	int nVal=SliderTriggerDelay->value();
	EditTriggerDelay->setText(tr("%1").arg(nVal));		
	int stat=ICubeSetCamParameter(nCurCam,REG_TRIGGER_DELAY,nVal);
	if(stat!=0)
		QMessageBox::information( 0,"Error"," REG_TRIGGER_DELAY failed",0);			
}
//------------------------------------------------------------------------
//
void DialogImpl::sliderStrobeDelaySlot(void)
{
	int nVal=SliderStrobeDelay->value();
	EditStrobeDelay->setText(tr("%1").arg(nVal));		
	int stat=ICubeSetCamParameter(nCurCam,REG_STROBE_DELAY,nVal);
	if(stat!=0)
		QMessageBox::information( 0,"Error"," REG_STROBE_DELAY failed",0);			
}
//------------------------------------------------------------------------
//
void DialogImpl::sliderStrobeLengthSlot(void)
{
	int nVal=SliderStrobeLength->value();
	EditStrobeLength->setText(tr("%1").arg(nVal));		
	int stat=ICubeSetCamParameter(nCurCam,REG_STROBE_LENGHT,nVal);
	if(stat!=0)
		QMessageBox::information( 0,"Error"," REG_STROBE_LENGHT failed",0);			
}
//------------------------------------------------------------------------
//


