#ifndef DIALOGIMPL_H
#define DIALOGIMPL_H

#include <QDialog>
#include "ui_Dialog.h"
#include <QImage>

#include "VideoControlsDlg.h"
#include "ViewModeDlg.h"

#include "glwidget.h"			// open gl preview
#include "ViewWindow.h"			// qt label preview
#include "HelperFunc.h"


#define GRAPHIC_MODE_QT	0		// Non OpenGl Preview (preview in Qlabel)
#define GRAPHIC_MODE_GL	1		// OpenGl Preview

 struct symbolic_list {
	const char *name;
};


class DialogImpl : public QDialog, public Ui::Dialog
{
Q_OBJECT
public:
	DialogImpl( QWidget * parent = 0, Qt::WFlags f = 0 );
	~DialogImpl();
     
	bool bSaveRaw;
	bool bPreview;
	bool bIsOpen;
	bool bStarted;
	int nCallbackType;
	int nTriggerMode;
	int nGraphicMode;
	unsigned int imgsize;
	
	unsigned int g_ModIndexCam[10];
	unsigned int g_iCurrentMode;
	
    static int ImageComplete(void* buffer, unsigned int bufferSize, void* context);
	void updatePreview(uchar *pBuf,ulong lBufferSize);

signals:
  void PreviewFromCallback(void);
  
private:	
	int initPreview(int nWidth,int nHeigth,int nCallbackformat);
	void releasePreview(void);
	int StopCam(void);
	void CloseCam(void);
	void StartCam(void);		
	int initTriggerControls(void);
	
	glWindow *GLpreviewWindow;				// OpenGl Preview 
	PreviewWindow *previewWindow;			// Non OpenGl Preview (preview in Qlabel)

	VideoControlsDlg *videoControlsDlg;
	ViewModeDlg 	 *viewModeDlg;	
	CFpsCounter *FpsCounter;
	CFpsCounter *FpsRefreshCounter;

	unsigned char* pBuffer;    
	
	unsigned long cntFr;
	double dFps;
	double dFpsRefresh;


protected:
	void closeEvent(QCloseEvent *event);       

public slots:
	void StopSlot(void);
	void RestartSlot(void);		
	
private slots:
	void OpenSlot(void);	
	void CloseSlot(void);
	void StartSlot(void);
	void SaveRawSlot(void);
	void updateCaption(void);
	void PreviewSlot(void);
	void bPreviewSlot(void);
	void ModeSlot(int nModeIndex);
	void FlipHSlot(void);
	void FlipVSlot(void);
	void VideoControlSlot(void);
	void ViewModeSlot(void);
	void r8bitSlot(void);
	void r8bitYSlot(void);	
	void r24bitSlot(void);
	void TrigEnSlot(void);
	void TrigSWSlot(void);
	void TrigHWSlot(void);
	void TrigDoSWSlot(void);
	void TrigInvertHWSlot(void);
	void GraphicQTSlot(void);
	void GraphicGLSlot(void);
	void SaveRGBSlot(void);
	void CamSlot(int nCamIndex);
	void sliderTriggerDelaySlot(void);
	void sliderStrobeDelaySlot(void);
	void sliderStrobeLengthSlot(void);

};


#endif



